#include "DirCodingRenderer.h"
#include "VectorAttributes.h"
#include "Grid.h"
#include <GLUT/glut.h>									//GLUT library
#include <algorithm>
#include <math.h>
#include <iostream>

using namespace std;



void DirCodingRenderer::draw(Grid& g)					//Draw 'g' by showing C0-interpolated vector-directions over cells
{
	glClearColor(1,1,1,1);								//0.   Clear the frame and depth buffers
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);			//1.   Render cells filled
	glDisable(GL_LIGHTING);
	glShadeModel(GL_SMOOTH);							//2.   Use bilinear interpolation when drawing cells as polygons		

	VectorAttributes& vectors = g.cellVectors();		//3.   Get cell vector attributes. We'll next compute and color-code their direction

	for(int i=0;i<g.numCells();++i)						//4.   Draw all cells in the grid filled bilinearly-interpolated polygons
	{
		int cell[10];									//Should have enough storage space for our largest cell
		
		int nvertices = g.getCell(i,cell);				//4.1. Get the 'nvertices' vertex-IDs of i-th cell
		if (nvertices!=3 && nvertices!=4)				//     We only handle here drawing of triangle and quad cells.
		   continue;									//     It is quite simple to extend this to other cell types.

		if (nvertices==3)								//     Triangle cells:
			glBegin(GL_TRIANGLES);						
		else //nvertices==4	
			glBegin(GL_QUADS);							//     Quad cells:
		
		Point3d v = vectors.getC0Vector(i);				//4.2. Get cell-vector data    

		float R,G,B;									//4.3. Map vector's direction to a color.		
		R = fabs(v.x);
		G = fabs(v.y);
		B = fabs(v.z);
			
		glColor3f(R,G,B);								//4.4. Pass cell color to OpenGL

		
		for(int j=0;j<nvertices;++j)
		{
			float x[2];
			g.getPoint(cell[j],x);						//4.5. Get vertex coordinates of j-th vertex of i-th cell							
			glVertex3f(x[0],x[1],0);					//4.6. Pass vertex coordinate to OpenGL
		}	

		glEnd();                                        //4.7. Finish rendering current cell
	}
	
	glutSwapBuffers();
}


