#pragma once

#include "Volume.h"
#include <map>
#include <string>



template<class T> class TensorVolume			//Implements a 3D tensor volume dataset. The volume stores the 2nd order
{												//partial derivatives of the basis signal, and also a confidence field.
public:											
  
					TensorVolume();
  void				makeVolume(int sizeX,int sizeY,int sizeZ);	

  int				getWidth() const 
					{ return sizeX; }

  int				getHeight()const  
					{ return sizeY; }

  int				getDepth() const 
					{ return sizeZ; }
					
  bool				readNRRD(const char* headerfile);	//Read volume from file in NRRD format

  Volume<T>&		component(int i)					//Convenience methods for accessing the tensor field's
					{ return *fields[i]; }				//components. These are ordered as:
														// confidence,dxx,dxy,dxz,dyy,dyz,dzz;
  const Volume<T>&	component(int i) const
					{ return *fields[i]; }

  Volume<T>*		component(const char* name);		//Retrieve component by name. If none exists for that name, returns 0

  const Volume<T>*	component(const char* name) const;	//Retrieve component by name. If none exists for that name, returns 0


  Volume<T>			confidence;							//Confidence we have, at each voxel, about the d** fields.
  Volume<T>			dxx,dxy,dxz,dyy,dyz,dzz;			//Second-order partial derivatives that form the tensor field.	
  
private:

  static const int  NumComponents = 7;					//Total number of scalar volumes this field is made of
  Volume<T>*		fields[NumComponents];				//Uniform way to access all stored tensor components
  std::map<std::string,Volume<T>* >						//Allows accessing the tensor components by name
					namedFields;

  int				sizeX,sizeY,sizeZ;

  bool				readHeader(const char* headerfile,char* data_file,bool& big_endian);	
														//Read NRRD header file
  bool				readData(const char* datafile,bool big_endian);		
														//Read NRRD raw data file	
};




